!+ 
! Module: omgrismplot_mod
! 
! Authors:  Vladimir Yershov - MSSL
!
! Description: PLOTS OM GRISM SPECTRA PRODUCED BY OMGRISM
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!- 
! VNY 27.10.2003 First version  
! VNY 07.11.2003 new parameter (plotflux) introduced
! VNY 20.11.2002 unused variables removed
! VNY 03.12.2003 the warning message about negative count rates is removed
! VNY 04.12.2003 Flux plotting included
! VNY 02.02.2004 Spectra ID's included ito the plot
! VNY 04.03.2004 A small bug is fixed (related to wrong data rebinning) 
! VNY 11.05.2004 TIMEDEL keyword is replaced with DLAMBDA in the header of the file              
! VNY 12.05.2004 Protected against old files that have TIMEDEL keyword
!                instead of DLAMBDA
! VNY 13.07.2004 using the new keyword OBJ_TYPE for labelling the target page
! VNY 02.10.2004 masking the contaminating sources is passed from omgrim 
!                to omgrismplot to avoid data loss in the output of omgrism
! VNY 03.06.2005 The task failure on the Full-Frame Low-Resolution mode with 
!                multiple spectra is fixed. The task was crashing due to unexplicable
!                behaviour of Fortran: by coding the same thing in a different way
!                the problem has gone   
!
module omgrismplot_mod

  use dal
  use param
  use ErrorHandling
  use metatask_mod
  use dss
  
  implicit none

  type(TaskInfo) :: info ! type from metatask_mod

  type test
     real(kind=single) :: Stat , Prob
  end type test

  type data_tests
     real(kind=single) :: Mean_Count , Variance
     type(test) :: Chi_Square , K_S
  end type data_tests

  private 

  public omgrismplot

contains

  !----------------
  subroutine omgrismplot
  !----------------
     
    character(len=128) :: Input_File , Output_File , Device_Type
    integer :: Binning_Factor, Plot_Flag
    real(kind=single) :: Bin_Width
    real(kind=single),dimension(:),pointer :: wl_Vect
    real(kind=single),dimension(:,:),pointer :: Net_Rate , Net_Err , Bkg_Rate , Bkg_Err
    real(kind=single),dimension(:,:),pointer :: Net_Count, Net_Total
    real(kind=single),dimension(:,:),pointer :: Flux, Flux_Err
    real(kind=single),dimension(:,:),pointer :: Net_Channel
    integer, dimension(:), pointer :: nFoundFeatures
    logical,dimension(:),pointer :: Good_Bin
    character(len=30) :: Content
    character(len=1000),dimension(5) :: Label
    integer :: Nlight , Nbin
    character(len=12) :: backapp
    real :: leftval, rightval
    integer :: nBlks, iBlk
    logical :: scaleBkgPlot
    integer :: featureWindow
    type(DataSetT) :: Ts_Set

    Input_File = StringParameter("set")
    Device_Type = change_case(StringParameter("plotdevice"),2)
    Output_File = StringParameter("plotfile")
    Binning_Factor = IntParameter("binsize")
    Plot_Flag = IntParameter("plotflux")
    scaleBkgPlot = booleanParameter("scalebkgplot")
    if (Binning_Factor .eq. 0) Binning_Factor=1

    Ts_Set = dataSet(Input_File,READ)
    nBlks=numberOfBlocks(Ts_Set)
    if (nBlks .gt. 0) then
       call open_plot(Device_Type , Output_File)
       do iBlk=1,nBlks
          call read_file (Input_File, Ts_Set, iBlk, Binning_Factor, Bin_Width, wl_Vect, &
               Net_Rate, Net_Err, Bkg_Rate, Bkg_Err, Flux, Flux_Err, Net_Channel, Nlight, Nbin, Label, &
               Content, backapp, leftval, rightval, nFoundFeatures, featureWindow)
          call rebin(Binning_Factor  , Bin_Width , wl_Vect , Net_Rate , Net_Err , &
               Bkg_Rate , Bkg_Err , Flux, Flux_Err, Good_Bin , Nlight , Nbin)
          call MaskZeroFeatures(Net_Rate, Bkg_Rate, Flux, Net_Channel, nFoundFeatures, Nlight, Nbin, featureWindow)
          call pgpage
          call draw_plot(Content, backapp, Label, wl_Vect, Net_Rate, Net_Err, &
               Bkg_Rate , Bkg_Err , Flux, Flux_Err, Nlight , Nbin, leftval, &
               rightval, Plot_Flag, nFoundFeatures)
          call recover_counts(Bin_Width*Binning_Factor , Net_Rate , Bkg_Rate, &
               Net_Count , Net_Total , Good_Bin , Nlight , Nbin)
          deallocate(wl_Vect) ; deallocate(Good_Bin)
          deallocate(Net_Rate , Net_Err , Bkg_Rate , Bkg_Err)
       enddo
       deallocate(Net_Count)
    endif
    call pgend
    call release(Ts_Set)
  end subroutine omgrismplot

  !-------------------------------------------------------------------------------------
  subroutine read_file(Input_File, Ts_Set, iBlk, Binning_Factor, Bin_Width, wl_Vect, Net_Rate, &
       Net_Err , Bkg_Rate , Bkg_Err , Flux, Flux_Err, Net_Channel, Nlight , Nbin , Label , Content, backapp, &
       leftval, rightval, nFoundFeatures, featureWindow)
  !-------------------------------------------------------------------------------------
  ! Reads source spectral series FITS file and recovers all data needed. Also creates wavelength
  ! vector.

    type(DataSetT),intent(in) :: Ts_Set
    character(len=*),intent(in) :: Input_File
    real(kind=single),intent(out) :: Bin_Width
    integer :: featureWindow
    real(kind=single),dimension(:),pointer :: wl_Vect
    real(kind=single),dimension(:,:),pointer :: Net_Rate , Net_Err , Bkg_Rate , Bkg_Err 
    real(kind=single),dimension(:,:),pointer :: Flux, Flux_Err, Net_Channel
    integer,dimension(:),pointer :: nFoundFeatures
    character(len=1000),dimension(5),intent(out) :: Label
    character(len=*),intent(out) :: Content
    integer,intent(in) :: iBlk
    integer,intent(out) :: Nlight , Nbin
    character(len=12) :: backapp
    integer :: Binning_Factor
    real :: leftval, rightval

  ! Local variables are Set, Table, Column and Pointer needed by DAL to recover FITS
  ! binary table. Moreover Time_Zero contains the start exposure time in MJD

    type( TableT ) :: Ts_Table
    type( ColumnT ) :: Col_Rate , Col_Error , Col_Backv , Col_Backe
    type(ColumnT) :: Col_Flux, Col_Flux_Error

    type(ColumnT) :: Col_Channel
    real(kind=single),dimension(:,:),pointer :: Array_Rate , Array_Error
    real(kind=single),dimension(:,:),pointer :: Array_Backv , Array_Backe, Array_Channel
    real(kind=single),dimension(:,:),pointer :: Array_Flux, Array_Flux_Error
    real(kind=single) :: Exposure_Time
    real(kind=single) :: Time_Zero
    integer :: i,j
    real(kind=single) :: iStep

    call read_data (Ts_Set, Content, backapp)

    call read_table (Input_File , Ts_Set , Ts_Table , &
         Time_Zero , Bin_Width , Exposure_Time, iBlk)

    Bin_Width=1.0
    Col_Channel = Column(Ts_Table, ColumnNumber(Ts_Table, 'WAVELENGTH'), Read)
    Col_Rate = Column(Ts_Table , ColumnNumber(Ts_Table,'RATE'),Read)
    Col_Error = Column(Ts_Table , ColumnNumber(Ts_Table,'ERROR'),Read)
    Col_Backv = Column(Ts_Table , ColumnNumber(Ts_Table,'BACKV'),Read)
    Col_Backe = Column(Ts_Table , ColumnNumber(Ts_Table,'BACKE'),Read)
    Col_Flux = Column(Ts_Table , ColumnNumber(Ts_Table,'FLUX'),Read)
    Col_Flux_Error = Column(Ts_Table , ColumnNumber(Ts_Table,'FLUX_ERR'),Read)

    Array_Channel => real32VectorData(Col_Channel)
    Array_Rate => real32VectorData(Col_Rate)
    Array_Error => real32VectorData(Col_Error)
    Array_Backv => real32VectorData(Col_Backv)
    Array_Backe => real32VectorData(Col_Backe)
    Array_Flux => real32VectorData(Col_Flux)
    Array_Flux_Error => real32VectorData(Col_Flux_Error)


    allocate( Net_Channel( size(Array_Channel(:,0)), size(Array_Channel(0,:))))
    allocate( Net_Rate( size(Array_Rate(:,0)) , size(Array_Rate(0,:)) ) )
    allocate( Net_Err( size(Array_Error(:,0)) , size(Array_Error(0,:)) ) )
    allocate( Bkg_Rate( size(Array_Backv(:,0)) , size(Array_Backv(0,:)) ) )
    allocate( Bkg_Err( size(Array_Backe(:,0)) , size(Array_Backe(0,:)) ) )
    allocate( Flux( size(Array_Flux(:,0)) , size(Array_Flux(0,:)) ) )
    allocate( Flux_Err( size(Array_Flux_Error(:,0)) , size(Array_Flux_Error(0,:)) ) )
    allocate(nFoundFeatures(size(Array_Rate(:,0))))

    Net_Rate = Array_Rate 
    Net_Err = Array_Error
    Bkg_Rate = Array_Backv ; Bkg_Err = Array_Backe
    Net_Channel=Array_Channel
    Flux=Array_Flux;
    Flux_Err=Array_Flux_Error;
    nFoundFeatures=0
!---

!---
    call create_labels (Input_File, Binning_Factor, Ts_Set , Ts_Table, &
    Label, leftval, rightval, iBlk, featureWindow)
    call release(Col_Channel)
    call release(Col_Rate)
    call release(Col_Error)
    call release(Col_Backv)
    call release(Col_Backe)
    call release(Col_Flux)
    call release(Col_Flux_Error)
    call release(Ts_Table)    

    ! More checks
    call check_sizes (Net_Rate , Net_Err , Bkg_Rate , Bkg_Err , Flux, &
         Flux_Err, Nlight , Nbin)    
    allocate( wl_Vect(Nbin) )
    iStep= size(Array_Channel(0,:))/real(Nbin)
    do i=1,Nbin
       j=int(real((i-1))*iStep)+1
       wl_Vect(i)=Net_Channel(1,j)
    end do
  end subroutine read_file

  !-------------------------------------------------------------------------------------
  subroutine read_data (Ts_Set, Content, backapp)
  !-------------------------------------------------------------------------------------
  ! Reads the in parameter FITS file and gives back a handle to corresponding dataset.
  ! Checks some keywords existence and validity.

    type( DataSetT ),intent(in) :: Ts_Set
    character(len=*),intent(out) :: Content
    character(len=12),intent(out) :: BACKAPP

    type( TableT ) :: Table1
    integer :: nBlks ! Number of blocks in the data set (number of spectra)
    nBlks=numberOfBlocks(Ts_Set)
    if (nBlks .gt. 0) then 
       Table1 = table(Ts_Set,0) 
    endif

    if (HasAttribute(Ts_Set,"BACKAPP") ) then
         backapp = StringAttribute(Ts_Set,'BACKAPP')
    else if (HasAttribute(Table1,"BACKAPP") ) then
         backapp = StringAttribute(Table1,'BACKAPP')
    else
         backapp = 'F'
    endif

    if ( .not. HasAttribute(Ts_Set,"CONTENT") ) then
      call warning('OMGRISMPLOT','Keyword CONTENT missing in the input file ')
        Content = 'SOURCE SPECTRUM'
    else
        Content = change_case(StringAttribute(Ts_Set,"CONTENT"),2)
       if ( ( Content /= 'OM GRISM SPECTRA' ) ) &
           call warning('OMGRISMPLOT','CONTENT parameter for input file '// &
                          ' is : '//Content)
    endif

    if (nBlks .gt. 0) then
       call release(Table1)
    endif
  end subroutine read_data

  !-------------------------------------------------------------------------------------
  subroutine read_table (Input_File, Ts_Set, Ts_Table, Time_Zero, Bin_Width, &
       Exposure_Time, iBlk )
  !-------------------------------------------------------------------------------------
  ! With FITS file name and handle to dataset, generates table.
  ! Checks more keywords.
  ! Also gives back Time_Zero, Bin_Width, Exposure_Time and Content.

    character(len=*),intent(in) :: Input_File
    type( DataSetT ),intent(in) :: Ts_Set
    type( TableT ),intent(out) :: Ts_Table
    integer, intent(in) :: iBlk

    real(kind=single),intent(out) :: Time_Zero, Bin_Width , Exposure_Time
    Ts_Table = table(Ts_Set,iBlk-1)

    if ( .not. HasAttribute(Ts_Table,"HDUCLASS") ) &
      call warning('OMGRISMPLOT','Keyword HDUCLASS missing in the input file '//Input_File)
    if ( change_case(StringAttribute(Ts_Table,"HDUCLASS"),2) /= 'OGIP' ) &
      call warning('OMGRISMPLOT','HDUCLAS is not equal to OGIP')
    if ( .not. HasAttribute(Ts_Table,"HDUCLAS1") ) &
      call warning('OMGRISMPLOT','Keyword HDUCLAS1 missing in the input file '//Input_File)
    if ( change_case(StringAttribute(Ts_Table,"HDUCLAS1"),2) /= 'SPECTRUM' ) &
      call warning('OMGRISMPLOT','HDUCLAS1 is not equal to SPECTRUM')
    if ( .not. HasAttribute(Ts_Table,"HDUCLAS2") ) &
      call warning('OMGRISMPLOT','Keyword HDUCLAS2 missing in the input file '//Input_File)
    if ( .not. HasAttribute(Ts_Table,"HDUCLAS3")) &
      call warning('OMGRISMPLOT','Keyword HDUCLAS3 missing in the input file '//Input_File)
    if ( change_case(StringAttribute(Ts_Table,"HDUCLAS3"),2) /= 'RATE' ) &
      call warning('OMGRISMPLOT','HDUCLAS3 is not equal to RATE')
    if (.not. HasAttribute(Ts_Table,"TSTART") ) then
      !call warning('OMGRISMPLOT','Keyword TSTART missing in the input file '//Input_File)
       Time_Zero = 0.0
    else
      Time_Zero = real32Attribute(Ts_Table,'TSTART')
    endif

    if ( .not. HasAttribute(Ts_Table,"DLAMBDA") ) then
       call warning('OMGRISMPLOT','Keyword DLAMBDA missing in the input file '//Input_File)
       if (.not. HasAttribute(Ts_Table, "TIMEDEL")) then
          Bin_Width=2.40774
       else
          Bin_Width=real32Attribute(Ts_Table, 'TIMEDEL')
       endif
    else
       Bin_Width = real32Attribute(Ts_Table,'DLAMBDA')
    endif
    if ( Bin_Width < epsilon(0.) ) then
       call warning('OMGRISMPLOT', 'Null or negative Bin_Width in the input file '//Input_File)
       Bin_Width=2.40774
    endif

    if ( .not. HasAttribute(Ts_Table,"EXPOSURE") ) then
         call warning('OMGRISMPLOT','Keyword EXPOSURE missing in the input file '//Input_File)
    else
         Exposure_Time = real32Attribute(Ts_Table,"EXPOSURE") 
         if ( Exposure_Time < epsilon(0.) ) call warning('OMGRISMPLOT', &
          'Null or negative exposure time in the input file '//Input_File)
    endif

  end subroutine read_table


  !-------------------------------------------------------------------------------------
  subroutine check_sizes(Net_Rate , Net_Err , Bkg_Rate , Bkg_Err , Flux, Flux_Err, Nlight , Nbin)
  !-------------------------------------------------------------------------------------
  ! Checks if all these vectors have same dimensions : Nlight * Nbin.

    real(kind=single),dimension(:,:),pointer :: Net_Rate , Net_Err , Bkg_Rate , Bkg_Err
    real(kind=single), dimension(:,:), pointer :: Flux, Flux_Err
    integer,intent(out) :: Nlight , Nbin

    Nlight = size(Net_Rate(:,1))
    if ( size(Net_Err(:,1)) /= Nlight ) call error('OMGRISMPLOT', &
         'Different number of columns between RATE, ERROR, BACKV and BACKE')
    if ( size(Bkg_Rate(:,1)) /= Nlight ) call error('OMGRISMPLOT', &
         'Different number of columns between RATE, ERROR, BACKV and BACKE')
    if ( size(Bkg_Err(:,1)) /= Nlight ) call error('OMGRISMPLOT', &
         'Different number of columns between RATE, ERROR, BACKV and BACKE')

    if ( size(Flux(:,1)) /= Nlight ) call error('OMGRISMPLOT', &
         'Different number of columns between RATE, ERROR, FLUX and FLUX_ERR')
    if ( size(Flux_Err(:,1)) /= Nlight ) call error('OMGRISMPLOT', &
         'Different number of columns between RATE, ERROR, FLUX  and FLUX_ERR')

    if ( Nlight > 9 ) call error('OMGRISMPLOT', &
         'Number of light-curves contained in a file must not exceed 9')
    if (.not. ( size(Net_Rate(1,:)) == size(Net_Err(1,:)) &
         .and. size(Bkg_Rate(1,:)) == size(Bkg_Err(1,:)) &
         .and. size(Net_Rate(1,:)) == size(Bkg_Rate(1,:)) )) call error('OMGRISMPLOT' ,&
         'Different number of rows in columns RATE, ERROR, BACKV and BACKE')
    Nbin = size( Net_Rate(1,:) )
    if ( Nbin <= 0 ) call error('OMGRISMPLOT','No data point received')

  end subroutine check_sizes 

  !-------------------------------------------------------------------------------------
  subroutine create_labels(Input_File, Binning_Factor, Ts_Set, Ts_Table, &
       Label, leftval, rightval, iBlk, featureWindow)

  !-------------------------------------------------------------------------------------
  ! Generates labels to add on plot.
 
    type( dataSetT ),intent(in) :: Ts_set
    character(len=1000),dimension(5),intent(out) :: Label
    integer, intent(in) :: iBlk
    integer, intent(out) :: featureWindow
    character(len=12) :: TELESCOP, INSTRUME, DETNAM, DATAMODE, OBSERVER, OBJECT, FILTER, FILT_COMMENT
    character(len=12) :: OBS_ID, OSW_ID
    character(len=12) :: EXPOSURE, SRCNO, PSF_CAPP, NPIXSOU, NPIXBACK
    character(len=12) :: EXTRACTIONMODE
    character(len=30) :: DATE_OBS, DATE_END,TSTART, EXP_ID, RA, DEC, PSF_FRAC
    character(len=70) :: Input_File 
    character(len=30) :: DLAMBDA, plotbinsizec 
    real(kind=double) :: TIME, plotbinsize


    integer :: Length, Binning_Factor
    real :: leftval, rightval
    type( TableT ),intent(in) :: Ts_Table
!
    character(len=1024) :: inputFileShortenName
    integer(kind=int16) :: posPoint
    character(len=10) :: blockName
    character(len=3) :: srcId
    integer :: extrMode ! spectrum extraction mode
    character(len=18) :: obj_type

    write(SRCNO,'(i3)') iBlk
    blockName=name(Ts_Table)
    srcId=blockName(5:7)
! VNY 13.07.2004
    if (HasAttribute(Ts_Set,"OBJ_TYPE")) then
       obj_type= StringAttribute(Ts_Set,'OBJ_TYPE')
    else if (HasAttribute(Ts_Table,"OBJ_TYPE")) then
       obj_type=StringAttribute(Ts_Table,'OBJ_TYPE')
    else
       obj_type='Indef'
    endif
!
    if (HasAttribute(Ts_Set,"EXP_ID") ) then
         EXP_ID = StringAttribute(Ts_Set,'EXP_ID')
    else
         EXP_ID = 'Indef'
    endif
    if (HasAttribute(Ts_Table,"RA_OBJ") ) then
         RA = StringAttribute(Ts_Table,'RA_OBJ')
    else if (HasAttribute(Ts_Table,"RA") ) then
         RA = StringAttribute(Ts_Table,'RA')
    else
         RA = 'Indef'
    endif
    if (RA .eq. '0') RA = 'Indef'
    if (HasAttribute(Ts_Table,"DEC_OBJ") ) then
         DEC = StringAttribute(Ts_Table,'DEC_OBJ')
    else if (HasAttribute(Ts_Table,"DEC") ) then
         DEC = StringAttribute(Ts_Table,'DEC')
    else
         DEC = 'Indef'
    endif
    if (DEC .eq. '0') DEC = 'Indef'

    if (HasAttribute(Ts_Set,"PSF_FRAC") ) then
         PSF_FRAC = StringAttribute(Ts_Set,'PSF_FRAC')
    else if (HasAttribute(Ts_Table,"PSF_FRAC") ) then
         PSF_FRAC = StringAttribute(Ts_Table,'PSF_FRAC')
    else
         PSF_FRAC = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"PSF_CAPP") ) then
         PSF_CAPP = StringAttribute(Ts_Set,'PSF_CAPP')
    else if (HasAttribute(Ts_Table,"PSF_CAPP") ) then
        PSF_CAPP = StringAttribute(Ts_Table,'PSF_CAPP')
    else
         PSF_CAPP = 'Indef'
    endif

    if (trim(PSF_CAPP) == 'T') then
      PSF_CAPP = 'YES'
    else if (trim(PSF_CAPP) == 'F') then
      PSF_CAPP = 'NO'
    else
      PSF_CAPP = 'Indef'
    end if

    if (HasAttribute(Ts_Set,"NPIXSOU") ) then
         NPIXSOU = StringAttribute(Ts_Set,'NPIXSOU')
    else if (HasAttribute(Ts_Table,"NPIXSOU") ) then
         NPIXSOU = StringAttribute(Ts_Table,'NPIXSOU')
    else
         NPIXSOU = 'Indef'
    endif
    if (trim(NPIXSOU) .eq. '0') NPIXSOU = 'Indef'

    if (HasAttribute(Ts_Set,"NPIXBACK") ) then
         NPIXBACK = StringAttribute(Ts_Set,'NPIXBACK')
    else if (HasAttribute(Ts_Table,"NPIXBACK") ) then
         NPIXBACK = StringAttribute(Ts_Table,'NPIXBACK')
    else
         NPIXBACK = 'Indef'
    endif
    if (trim(NPIXBACK) .eq. '0') NPIXBACK = 'Indef'

    if (HasAttribute(Ts_Set,"DLAMBDA") ) then
         TIME = real64Attribute(Ts_Set,'DLAMBDA')
         plotbinsize=TIME*real(Binning_Factor)
         call string_of_real2(plotbinsize, plotbinsizec, Length)
         call string_of_real2(TIME, DLAMBDA, Length)
    else if (HasAttribute(Ts_Table,"DLAMBDA") ) then
         TIME = real64Attribute(Ts_Table,'DLAMBDA')
         plotbinsize=TIME*real(Binning_Factor)
         call string_of_real2(plotbinsize, plotbinsizec, Length)
         call string_of_real2(TIME, DLAMBDA, Length)
    else
         DLAMBDA = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"EXTRMODE") ) then
       EXTRACTIONMODE=StringAttribute(Ts_Set,'EXTRMODE')
    else 
       if (HasAttribute(Ts_Table,"EXTRMODE") ) then
          EXTRACTIONMODE=StringAttribute(Ts_Table,'EXTRMODE')
       else
          EXTRACTIONMODE='Indef'
       endif
    endif

    if (HasAttribute(Ts_Set,"TSTART") ) then
         TIME = real64Attribute(Ts_Set,'TSTART')
         call string_of_real2(TIME, TSTART, Length)
    else if (HasAttribute(Ts_Table,"TSTART") ) then
         TIME = real64Attribute(Ts_Table,'TSTART')
         call string_of_real2(TIME, TSTART, Length)
    else
         TSTART = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"EXPOSURE") ) then
         EXPOSURE = StringAttribute(Ts_Set,'EXPOSURE')
    else if (HasAttribute(Ts_Table,"EXPOSURE") ) then
         EXPOSURE = StringAttribute(Ts_Table,'EXPOSURE')
    else
         EXPOSURE = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DATE-END") ) then
         DATE_END = StringAttribute(Ts_Set,'DATE-END')
    else if (HasAttribute(Ts_Table,"DATE-END") ) then
         DATE_END = StringAttribute(Ts_Table,'DATE-END')
    else
         DATE_END = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DATE-OBS") ) then
         DATE_OBS = StringAttribute(Ts_Set,'DATE-OBS')
    else if (HasAttribute(Ts_Table,"DATE-OBS") ) then
         DATE_OBS = StringAttribute(Ts_Table,'DATE-OBS')
    else
         DATE_OBS = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBS_ID") ) then
         OBS_ID = StringAttribute(Ts_Set,'OBS_ID')
    else if (HasAttribute(Ts_Table,"OBS_ID") ) then
         OBS_ID = StringAttribute(Ts_Table,'OBS_ID')
    else
         OBS_ID = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OSW_ID") ) then
         OSW_ID = StringAttribute(Ts_Set,'OSW_ID')
    else if (HasAttribute(Ts_Table,"OSW_ID") ) then
         OSW_ID = StringAttribute(Ts_Table,'OSW_ID')
    else
         OSW_ID = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBJECT") ) then
         OBJECT = StringAttribute(Ts_Set,'OBJECT')
    else if (HasAttribute(Ts_Table,"OBJECT") ) then
         OBJECT = StringAttribute(Ts_Table,'OBJECT')
    else
         OBJECT = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBSERVER") ) then
         OBSERVER = StringAttribute(Ts_Set,'OBSERVER')
    else if (HasAttribute(Ts_Table,"OBSERVER") ) then
         OBSERVER = StringAttribute(Ts_Table,'OBSERVER')
    else
         OBSERVER = 'Indef'
    endif
!--- Using information about the filter
   if (HasAttribute(Ts_Set,"FILTER")) then
      FILTER= StringAttribute(Ts_Set,'FILTER')
   else if (HasAttribute(Ts_Table,"FILTER")) then
      FILTER=StringAttribute(Ts_Table,'FILTER')
   else
      FILTER='Indef'
   endif
   FILT_COMMENT=' '
   leftval=1600
   rightval=7000
   featureWindow=200 ! the window for detetecting the contaminating features
   if (trim(FILTER) .eq. "Grism2" .or. trim(FILTER) .eq. "GRISM2") then
      FILT_COMMENT=' (Visual)'
     leftval=2800 ! left edge of the visual spectrum
     rightval=6800
     featureWindow=80 !100
   endif
   if (trim(FILTER) .eq. "Grism1" .or. trim(FILTER) .eq. "GRISM1") then
      FILT_COMMENT=' (UV)'
      leftval=1700
      rightval=3650
      featureWindow=200
   endif

   if (HasAttribute(Ts_Set,"DATAMODE") ) then
         DATAMODE = StringAttribute(Ts_Set,'DATAMODE')
    else if (HasAttribute(Ts_Table,"DATAMODE") ) then
         DATAMODE = StringAttribute(Ts_Table,'DATAMODE')
    else
         DATAMODE = ' '
    endif

    if (HasAttribute(Ts_Set,"DETNAM") ) then
         DETNAM = StringAttribute(Ts_Set,'DETNAM')
    else if (HasAttribute(Ts_Table,"DETNAM") ) then
         DETNAM = StringAttribute(Ts_Table,'DETNAM')
    else
         DETNAM = ' '
    endif

    if (HasAttribute(Ts_Set,"INSTRUME") ) then
         INSTRUME = StringAttribute(Ts_Set,'INSTRUME')
    else if (HasAttribute(Ts_Table,"INSTRUME") ) then
         INSTRUME = StringAttribute(Ts_Table,'INSTRUME')
    else
         INSTRUME = ' '
    endif

    if (HasAttribute(Ts_Set,"TELESCOP") ) then
         TELESCOP = StringAttribute(Ts_Set,'TELESCOP')
    else if (HasAttribute(Ts_Table,"TELESCOP") ) then
         TELESCOP = StringAttribute(Ts_Table,'TELESCOP')
    else
         TELESCOP = ' '
    endif
    posPoint=index(trim(Input_File), '/p', .true.)
    if (posPoint .gt. 0) then
       inputFileShortenName=Input_File(posPoint+1:1024)
    else
       inputFileShortenName=trim(Input_File)
    endif

    Label(1) = trim(TELESCOP)//' - '// &
               trim(INSTRUME)//' - '// &
               trim(DATAMODE)// &
               ' - Filename: '//trim(inputFileShortenName)
    Label(2)=" "
    if (trim(OBJECT) .ne. 'Indef') Label(2)=trim(Label(2)) &
         //'Object:'//trim(OBJECT)
    if (trim(OBS_ID) .ne. 'Indef') Label(2)=trim(Label(2)) &
         //' - Obs ID: '//trim(OBS_ID)
    if (trim(EXP_ID) .ne. 'Indef') Label(2)=trim(Label(2)) &
         //' - Exp ID: '//trim(EXP_ID)
    if (trim(OSW_ID) .ne. 'Indef') Label(2)=trim(Label(2)) &
         //' - OSW ID: '//trim(OSW_ID)
    if (trim(FILTER) .ne. 'Indef') Label(2)=trim(Label(2)) &
         //' - Filter: '//trim(FILTER)//trim(FILT_COMMENT)

    Label(3)=" "
    if (trim(SRCNO) .ne. 'Indef') Label(3)=trim(Label(3)) &
         //'SRCE No:'//trim(SRCNO)
    if (trim(srcId) .ne. 'Indef') Label(3)=trim(Label(3)) &
         //' - ID: '//trim(srcId)
    if (trim(RA) .ne. 'Indef') Label(3)=trim(Label(3)) &
         //' - RA: '//trim(RA)
    if (trim(DEC) .ne. 'Indef') Label(3)=trim(Label(3)) &
         //' - DEC: '//trim(DEC)
    if (trim(NPIXSOU) .ne. 'Indef') Label(3)=trim(Label(3)) &
         //' - SRCE REG: '//trim(NPIXSOU)// ' pix'
    if (trim(EXTRACTIONMODE) .ne. 'Indef') Label(3)=trim(Label(3)) &
         //' - Extr.Mode: '//trim(EXTRACTIONMODE)
    ! VNY 13.07.2004 Checking for the keyword OBJ_TYPE
    if (trim(obj_type) .ne. 'Indef') then 
       Label(3)=trim(Label(3)) &
            //' - Object: '//trim(obj_type)
    endif

    Label(4)=" "
    if (trim(NPIXBACK) .ne. 'Indef') Label(4)=trim(Label(4)) &
         //'BKG REG: '//trim(NPIXBACK)
    if (trim(PSF_FRAC) .ne. 'Indef') Label(4)=trim(Label(4)) &
         //' - PSF FRAC: '//trim(PSF_FRAC)
    if (trim(PSF_CAPP) .ne. 'Indef') Label(4)=trim(Label(4)) &
         //' - Vignet.Corr.: '//trim(PSF_CAPP)

    Label(5)=" "
    if (trim(DATE_OBS) .ne. 'Indef') Label(5)=trim(Label(5)) &
         //'Start: '//trim(DATE_OBS)
    if (trim(DATE_END) .ne. 'Indef') Label(5)=trim(Label(5)) &
         //' - Stop: '//trim(DATE_END)
    if (trim(EXPOSURE) .ne. 'Indef') Label(5)=trim(Label(5)) &
         //' - Exp.time: '//trim(EXPOSURE)//' secs'

  end subroutine create_labels

  !-------------------------------------------------------------------------------------
  subroutine rebin(Binning_Factor , Bin_Width , wl_Vect , Net_Rate , Net_Err , &
       Bkg_Rate , Bkg_Err , Flux, Flux_Err, Good_Bin , Nlight , Nbin)
  !-------------------------------------------------------------------------------------
  ! Rebins Net_Rate, Net_Err, Bkg_Rate, Bkg_Err by at least a factor five to ensure
  ! good signal-to-noise and to limit the number of plotted points to 500.
  ! Also modifies wl_Vect.

    integer,intent(inout) :: Binning_Factor
    real(kind=single),dimension(:,:),pointer :: Net_Rate , Net_Err , Bkg_Rate , Bkg_Err
    real(kind=single),dimension(:,:),pointer :: Flux , Flux_Err
    integer,intent(in) :: Nlight
    integer,intent(inout) :: Nbin
    real(kind=single),intent(in) :: Bin_Width
    real(kind=single),dimension(:),pointer :: wl_Vect
    logical,dimension(:),pointer :: Good_Bin

    real(kind=single),dimension(:,:),pointer :: Net_Rate_Reb , Net_Err_Reb
    real(kind=single),dimension(:,:),pointer :: Bkg_Rate_Reb , Bkg_Err_Reb
    real(kind=single),dimension(:,:),pointer :: Flux_Reb , Flux_Err_Reb
    real(kind=single),dimension(:),pointer :: wl_Reb
    real(kind=single),dimension(Nbin) :: wl_Vect_Temp
    real(kind=single),dimension(Nlight,Nbin) :: Net_Rate_Temp , Net_Err_Temp
    real(kind=single),dimension(Nlight,Nbin) :: Bkg_Rate_Temp , Bkg_Err_Temp
    real(kind=single),dimension(Nlight,Nbin) :: Flux_Temp , Flux_Err_Temp
    integer :: Nbin_Reb ,  i , j, Nbin_Temp, Light
    logical :: No_Gap, Good_Value
    real :: Eps, averr
    integer :: i1

    ! Sets Binning_Factor to correct value
    if (Binning_Factor < 1) then
       Binning_Factor = 1
       call warning('OMGRISMPLOT','Binning_Factor must be positive, set to default value : 1')
    end if
    if ( int(Nbin/Binning_Factor)>1800 ) then
       Binning_Factor = (Nbin-1)/1800 + 1
       write(errstr,*) 'Data point to plot must not exceed 1800. '//&
                  'Binning Factor is increased from input value to : ',Binning_Factor
       call warning('OMGRISMPLOT')
    end if

    ! Rebinning
    allocate(wl_Reb(Nbin))
    allocate(Good_Bin(Nbin))
    allocate(Net_Rate_Reb(Nlight,Nbin))
    allocate(Net_Err_Reb(Nlight,Nbin))
    allocate(Bkg_Rate_Reb(Nlight,Nbin))      
    allocate(Bkg_Err_Reb(Nlight,Nbin))   
    allocate(Flux_Reb(Nlight,Nbin))      
    allocate(Flux_Err_Reb(Nlight,Nbin))      
    Eps = Bin_Width/10.
    Nbin_Reb = 0
    i = 1
    !do while (i <= Nbin)
    ! VNY 03.06.05
    do while (i <= Nbin-1)
       Nbin_Reb                 =  Nbin_Reb + 1
       wl_Reb(Nbin_Reb)       =  wl_Vect(i)
       Net_Rate_Reb(:,Nbin_Reb) =  Net_Rate(:,i)
       Net_Err_Reb(:,Nbin_Reb)  =  Net_Err(:,i)**2
       Bkg_Rate_Reb(:,Nbin_Reb) =  Bkg_Rate(:,i)
       Bkg_Err_Reb(:,Nbin_Reb)  =  Bkg_Err(:,i)**2
       Flux_Reb(:,Nbin_Reb) =  Flux(:,i)
       Flux_Err_Reb(:,Nbin_Reb)  =  Flux_Err(:,i)**2
       i = i + 1 ; j = 1
       No_Gap=(abs(wl_Vect(min(i,Nbin))-wl_Vect(i-1)-Bin_Width) <= Eps)
       do while ( ( (i <= Nbin) .and. (j < Binning_Factor) ))
          Net_Rate_Reb(:,Nbin_Reb)  =  Net_Rate_Reb(:,Nbin_Reb) + Net_Rate(:,i)
          Net_Err_Reb(:,Nbin_Reb)   =  Net_Err_Reb(:,Nbin_Reb)  + Net_Err(:,i)**2
          Bkg_Rate_Reb(:,Nbin_Reb)  =  Bkg_Rate_Reb(:,Nbin_Reb) + Bkg_Rate(:,i)
          Bkg_Err_Reb(:,Nbin_Reb)   =  Bkg_Err_Reb(:,Nbin_Reb)  + Bkg_Err(:,i)**2
          Flux_Reb(:,Nbin_Reb)  =  Flux_Reb(:,Nbin_Reb) + Flux(:,i)
          Flux_Err_Reb(:,Nbin_Reb)   =  Flux_Err_Reb(:,Nbin_Reb)  + Flux_Err(:,i)**2
          j = j + 1 ; i = i + 1
          No_Gap=(abs(wl_Vect(min(i,Nbin))-wl_Vect(i-1)-Bin_Width) <= Eps)
       end do
       Net_Rate_Reb(:,Nbin_Reb)  =  Net_Rate_Reb(:,Nbin_Reb) / j
       Net_Err_Reb(:,Nbin_Reb)   =  sqrt(Net_Err_Reb(:,Nbin_Reb)) / j
       Bkg_Rate_Reb(:,Nbin_Reb)  =  Bkg_Rate_Reb(:,Nbin_Reb) / j
       Bkg_Err_Reb(:,Nbin_Reb)   =  sqrt(Bkg_Err_Reb(:,Nbin_Reb)) / j
       Flux_Reb(:,Nbin_Reb)  =  Flux_Reb(:,Nbin_Reb) / j
       Flux_Err_Reb(:,Nbin_Reb)   =  sqrt(Flux_Err_Reb(:,Nbin_Reb)) / j
       Good_Bin(Nbin_Reb)        =  ( j == Binning_Factor )
    end do
    deallocate(wl_Vect)
    deallocate(Net_Rate , Net_Err , Bkg_Rate , Bkg_Err, Flux, Flux_Err)
    Nbin = Nbin_Reb
    allocate(wl_Vect(Nbin))
    allocate(Net_Rate(Nlight,Nbin) , Net_Err(Nlight,Nbin) )
    allocate(Bkg_Rate(Nlight,Nbin) , Bkg_Err(Nlight,Nbin) )
    allocate(Flux(Nlight,Nbin) , Flux_Err(Nlight,Nbin) )
    do i1=1,Nbin
       wl_Vect(i1)=wl_Reb(i1)
       Net_Rate(:,i1)   =  Net_Rate_Reb(:,i1)
       Net_Err(:,i1)    =  Net_Err_Reb(:,i1)
       Bkg_Rate(:,i1)   =  Bkg_Rate_Reb(:,i1)
       Bkg_Err(:,i1)    =  Bkg_Err_Reb(:,i1)
       Flux(:,i1)   =  Flux_Reb(:,i1)
       Flux_Err(:,i1)    =  Flux_Err_Reb(:,i1)
    enddo

!---------------
    deallocate(Net_Rate_Reb , Net_Err_Reb , Bkg_Rate_Reb , Bkg_Err_Reb, &
         Flux_Reb, Flux_Err_Reb)
  end subroutine rebin

!-------------------------------------------------------------------------------------
  subroutine open_plot(Device_Type, Output_File) 
  !-------------------------------------------------------------------------------------
  ! Check Device_Type and Output_File. Open plot if required.
    character(len=*),intent(inout) :: Device_Type , Output_File
    integer :: pgbeg
    character(len=2048) :: Device_Selected
    logical :: File_Exists

    if (Output_File == '')  Output_File='default.ps'
    if (Device_Type(1:3) /= '/XW') then
         inquire(file=Output_File,exist=File_Exists)   
         if (File_Exists .and. .not. clobber()) call error('OMGRISMPLOT', &
         'Already existing file '//trim(Output_File)//' cannot be overwritten (SAS_CLOBBER=0)')
         if (File_Exists .and. clobber()) call warning('OMGRISMPLOT', &
         'Already existing file '//trim(Output_File)//' will be overwritten')
         Device_Selected=trim(Output_File)//trim(Device_Type)
         if ( pgbeg(0,Device_Selected,1,1)/=1 ) then
             call warning('OMGRISMPLOT', &
            'Invalid device and/or output file : '//trim(Device_Type)//' , '//trim(Output_File))
             Output_File='default.ps'
!             Device_Type= '/PS'
             Device_Type= '/VC'
             Device_Selected=trim(Output_File)//trim(Device_Type)
             if ( pgbeg(0,Device_Selected,1,1)/=1 ) then
                  call error('OMGRISMPLOT', &
                  'Invalid DEFAULT device and/or output file : '//trim(Device_Type)//' , '&
                  //trim(Output_File))
             else
                  call warning('OMGRISMPLOT', &
                 'New device and output file : '//trim(Device_Type)//' , '//trim(Output_File))
            endif
         endif
    else
        Device_Selected=trim(Device_Type)
        if ( pgbeg(0,Device_Selected,1,1)/=1 ) call error('OMGRISMPLOT', &
        'Invalid device : '//trim(Device_Type))
   endif
   call pgpap(0.0 , 1.0 )
   call pgask(.true.)
   call pgsch(0.9)

  end subroutine open_plot

  !-------------------------------------------------------------------------------------
  subroutine draw_plot(Content, backapp, Label, wl_Vect, Net_Rate, Net_Err,&
       Bkg_Rate , Bkg_Err , Flux, Flux_Err, Nlight , Nbin, leftval, rightval, &
       Plot_Flag, nFoundFeatures)
  !-------------------------------------------------------------------------------------
  ! Draw labelled plot. A Nlight pages one, in which each page contains the net and
  ! background source spectra serie representation of a specific spectrum.

    character(len=*),intent(in) :: Content
    character(len=1000),dimension(5),intent(in) :: Label
    real(kind=single),dimension(:),pointer :: wl_Vect
    real(kind=single),dimension(:,:),pointer :: Net_Rate , Net_Err , Bkg_Rate , Bkg_Err
    real(kind=single), dimension(:,:),pointer :: Flux, Flux_Err
    integer, dimension(:), pointer :: nFoundFeatures
    character(len=1000) :: labelFoundFeatures
    integer,intent(in) :: Nlight , Nbin, Plot_Flag
    integer :: whichfile
    real :: botval, topval, leftval, rightval
    character(len=12) :: backapp

  ! Local variables Window1&2 are used to define the viewports.
    real(kind=single),dimension(4) :: Window1 , Window2
    integer :: Light , i
    real :: sizeRate

    info = getTaskInfo()
    Window1 = (/0.1,0.95,0.480,0.85/)
    Window2 = (/0.1,0.95,0.1,0.33/)

    do Light=1,Nlight
       if (Plot_Flag .ne. 1) then
          call pgpage
          call pgsvp(0.0,1.0,0.0,1.0)
          call pgsci(11)
          call pgsch(0.6)
          call pgmtxt("b", -1.0, 0.01, 0.0, trim(nameOf(info))//' v'//trim(versionOf(info)))

          do i=1,3
             call pgmtxt('T' , -(1. + (i-1)*1.2) , 0.1 , 0.0 , Label(i))
          end do  
          i=5
          call pgmtxt('T' , -(1. + (i-2)*1.2) , 0.1 , 0.0 , Label(i))
          if (nFoundFeatures(Light) .gt. 0) then
             write(labelFoundFeatures,*) &
                  "NB: The spectrum is likely to be contaminated by zero-order features (number of features:", &
                  nFoundFeatures(Light), ")"
             call pgmtxt('T' , -(1. + (i-1)*1.2) , 0.1 , 0.0 , trim(labelFoundFeatures))
          endif
          call pgsci(11)
          call pgsch(0.9)
          whichfile=1
          call make_one_plot(trim(Content) , backapp, Window1 , wl_Vect , &
               Net_Rate(Light,:) , Net_Err(Light,:) , Nbin, whichfile, botval, topval, &
               leftval, rightval)
          call pgsch(0.9)
          whichfile=2
          call make_one_plot('BACKGROUND RATES' , backapp, Window2 , wl_Vect , &
               Bkg_Rate(Light,:) , Bkg_Err(Light,:) , Nbin, whichfile, botval, topval, &
               leftval, rightval)
       endif ! Plot_Flag .ne. 1

       if (Plot_Flag .ne. 0) then

          call pgpage
          call pgsvp(0.0, 1.0, 0.0, 1.0)
          call pgsci(11)
          call pgsch(0.6)
          call pgmtxt("b", -1.0, 0.01, 0.0, trim(nameOf(info))//' v'//trim(versionOf(info)))
          do i=1,3
             call pgmtxt('T' , -(1. + (i-1)*1.2) , 0.1 , 0.0 , Label(i))
          end do
          i=5
          call pgmtxt('T' , -(1. + (i-2)*1.2) , 0.1 , 0.0 , Label(i))
          call pgsci(11)
          call pgsch(0.9)       
          whichfile=3          
          call make_one_plot('FLUX' , backapp, Window1 , wl_Vect , &
               Flux(Light,:) , Flux_Err(Light,:) , Nbin, whichfile, botval, topval, &
               leftval, rightval)
       endif ! Plot_Flag .ne. 0
    end do

  end subroutine draw_plot

  !-------------------------------------------------------------------------------------
  subroutine make_one_plot(Title, backapp, Window, wl_Vect, Rate, Err , &
       Nbin, whichfile, botval, topval, leftval, rightval)
  !-------------------------------------------------------------------------------------
  ! Draw one specific plot. X-axis corresponds to wavelength and Y-axis to count rates.
  ! Plot title and mean rate are labelled.

    character(len=*) :: Title
    real(kind=single),dimension(4),intent(in) :: Window
    real(kind=single),dimension(:),pointer :: wl_Vect
    real(kind=single),dimension(:),intent(in) :: Rate , Err
    real(kind=single), dimension(:),pointer :: modRate, modErr
    real :: botval, topval, leftval, rightval
    integer,intent(in) :: Nbin, whichfile
    character(len=12) :: backapp
    character(len=20) :: String, StringE
    character(len=20) :: StringF, StringFE
    integer :: Length, LengthE, LengthF, LengthFE
    integer :: i
    integer sizeRate
    real :: Nbin2


    sizeRate=size(Rate)
    allocate(modRate(1:sizeRate))
    allocate(modErr(1:sizeRate))
    modRate=Rate
    modErr=Err
    Nbin2=0.0
    do i=1, sizeRate
       if (wl_Vect(i) .lt. leftval .or. wl_Vect(i) .gt. rightval) then
          modRate(i)=0.0
          modErr(i)=0.0
          nBin2=Nbin2+1.0
       endif
    enddo

    if (Nbin2 .gt. 1.0) then 
       call string_of_real(sum(modRate)/real(Nbin2) , String , Length )
       call string_of_real(sum(modErr)/real(Nbin2), StringE, LengthE)
    else
       call string_of_real(sum(modRate)/real(Nbin) , String , Length )
       call string_of_real(sum(modErr)/real(Nbin), StringE, LengthE)
    endif
!  string for priting average value for the error bars
    call pgsvp(Window(1),Window(2),Window(3),Window(4))
    if (Nbin2 .gt. 1.0) then
       write(stringF,*) sum(modRate)/real(Nbin2)
    else
       write(stringF,*) sum(modRate)/real(Nbin)
    endif
    stringF=trim(stringF)
    LengthF=len(stringF)
    if (Nbin2 .gt. 1.0) then
       write(stringFE, *) sum(modErr)/real(Nbin2)
    else
       write(stringFE, *) sum(modErr)/real(Nbin)
    endif
    stringFE=trim(stringFE)
    LengthFE=len(stringFE)

    !botval=min(0.,minval(Rate))
    botval=minval(modRate)
    topval=maxval(modRate)
    if (whichfile == 1) then
       botval=minval(modRate)
       call pgswin(leftval, rightval, &
            botval , maxval(modRate) )
    else if  (whichfile == 2 .and. .not. booleanParameter("scalebkgplot")) then
       call pgswin(leftval, rightval, &
         botval , maxval(modRate) )
    else if (whichfile == 2 .and. booleanParameter("scalebkgplot")) then
       call pgswin(leftval, rightval, &
         0.0, topval )
    else if (whichFile == 3 .and. .not. booleanParameter("scalebkgplot")) then
       call pgswin(leftval, rightval, botval, topval)
    else
       call pgswin(leftval, rightval, &
         0. , maxval(modRate) )
    end if      
    call pgsci(1)
    call pgsch(0.6)
    call pgbox('BCNST' , 0.0 , 20 ,'BCNST' , 0.0 , 5)
    if (trim(backapp) == 'T' .and. Title /= 'BACKGROUND RATES') then
       if (Title == 'FLUX') then
          call pglab('Wavelength (Angstroms)', 'Erg/sec/sm**2/Anstrom', & 
        Title//', BACKGROUND SUBTRACTED - Mean value='// StringF(1:LengthF) // &
        ' - Aver.Err.='//StringFE(1:LengthFE))
       else
          call pglab('Wavelength (Angstroms)', 'Counts/sec/Anstrom', & 
               Title//', BACKGROUND SUBTRACTED - Mean value='// String(1:Length) // &
               ' - Aver.Err.='//StringE(1:LengthE))
       endif
    else 
      call pglab('Wavelength (Angstroms)', 'Counts/sec/Angstrom', & 
         Title//' - Mean value='// String(1:Length) // &
         ' - Aver.Err.='//StringE(1:LengthE))
    end if
    call pgsci(2)
    call pgsci(1) ! Color index
    call pgslw(3) ! Linewidth
     call pgline(Nbin, wl_Vect, modRate)
    call pgslw(1)
    deallocate(modRate)
    deallocate(modErr)
  end subroutine make_one_plot

!*****************************
  !-------------------------------------------------------------------------------------
  subroutine draw_plot_unrotated(Content, backapp, Label, wl_Vect, Net_Rate, Net_Err,&
       Bkg_Rate , Bkg_Err , Nlight , Nbin, leftval, rightval)
  !-------------------------------------------------------------------------------------
  ! Draw labelled plot. A Nlight pages one, in which each page contains the net and
  ! background source spectra serie representation of a specific spectrum.

    character(len=*),intent(in) :: Content
    character(len=1000),dimension(5),intent(in) :: Label
    real(kind=single),dimension(:),pointer :: wl_Vect
    real(kind=single),dimension(:,:),pointer :: Net_Rate , Net_Err , Bkg_Rate , Bkg_Err
    integer,intent(in) :: Nlight , Nbin
    integer :: whichfile
    real :: botval, topval, leftval, rightval
    character(len=12) :: backapp

  ! Local variables Window1&2 are used to define the viewports.

    real(kind=single),dimension(4) :: Window1 , Window2
    integer :: Light , i

    info = getTaskInfo()
    Window1 = (/0.1,0.95,0.480,0.85/)
    Window2 = (/0.1,0.95,0.1,0.33/)

    do Light=1,Nlight
       call pgpage
       call pgsvp(0.0,1.0,0.0,1.0)
       call pgsci(11)
       call pgsch(0.6)
       call pgmtxt("b", -1.0, 0.01, 0.0, trim(nameOf(info))//' v'//trim(versionOf(info)))

       do i=1,3
          call pgmtxt('T' , -(1. + (i-1)*1.2) , 0.1 , 0.0 , Label(i))
       end do
       i=5
       call pgmtxt('T' , -(1. + (i-2)*1.2) , 0.1 , 0.0 , Label(i))
       call pgsci(11)
       call pgsch(0.9)
       call pgsch(0.9)
       whichfile=1
       call make_one_plot(trim(Content) , backapp, Window1 , wl_Vect , &
            Net_Rate(Light,:) , Net_Err(Light,:) , Nbin, whichfile, botval, topval, &
            leftval, rightval)
       call pgsch(0.9)
       whichfile=2
       call make_one_plot('BACKGROUND RATES' , backapp, Window2 , wl_Vect , &
            Bkg_Rate(Light,:) , Bkg_Err(Light,:) , Nbin, whichfile, botval, topval, &
            leftval, rightval)
    end do

  end subroutine draw_plot_unrotated
!*****************************
  !----------------------------------------------------------------------------------
  subroutine recover_counts(wl_Length , Net_Rate , Bkg_Rate , Net_Count , &
       Net_Total, Good_Bin , Nlight , Nbin)
  !----------------------------------------------------------------------------------
  ! Recovers total source counts (net*wl)

    real(kind=single),intent(in) :: wl_Length
    real(kind=single),dimension(:,:),pointer :: Net_Rate , Net_Count
    real(kind=single),dimension(:,:),pointer :: Bkg_Rate , Net_Total
    real(kind=single),dimension(:,:),pointer :: Net_Count_Temp, Net_Total_Temp
    logical,dimension(:),pointer :: Good_Bin
    integer,intent(in) :: Nlight
    integer,intent(inout) :: Nbin
    integer :: i, n

    allocate(Net_Count_Temp(Nlight,Nbin))
    allocate(Net_Total_Temp(Nlight,Nbin))
    n=0
    do i=1,Nbin
       if ( Good_Bin(i) ) n = n + 1
    end do
    do i=1,Nbin
       if ( Good_Bin(i) ) then
          Net_Count_Temp(:,i) = Net_Rate(:,i) * wl_Length
          Net_Total_Temp(:,i) = (Net_Rate(:,i)+Bkg_Rate(:,i)) * wl_Length
       endif
    end do
    Nbin = n
    allocate(Net_Count(Nlight,Nbin))
    allocate(Net_Total(Nlight,Nbin))
    Net_Count = Net_Count_Temp(:,1:Nbin)
    Net_Total = Net_Total_Temp(:,1:Nbin)

    deallocate(Net_Count_Temp)
    deallocate(Net_Total_Temp)

  end subroutine recover_counts



  !-------------------------------------------------------------------------------------
  subroutine string_of_real(Nb , String , Length)
  !-------------------------------------------------------------------------------------
  ! Converts real numbers into plottable character chain. Gives also its length.

    real(kind=single),intent(in) :: Nb
    character(len=20),intent(out) :: String
    integer,intent(out) :: Length
    integer :: MM , puiss

    puiss = max( 0 , min( 8 , 8-int(log10(Nb)) ) )
    MM = nint(Nb*(10**puiss))
    call pgnumb(MM , -puiss , 0 , String , Length)

  end subroutine string_of_real

  !-------------------------------------------------------------------------------------
  subroutine string_of_real2(Nb , String , Length)
  !-------------------------------------------------------------------------------------
  ! Converts real numbers into plottable character chain. Gives also its length.

    real(kind=double),intent(in) :: Nb
    character(len=30),intent(out) :: String
    integer,intent(out) :: Length
    integer :: MM , puiss

    puiss = max( 0 , min( 9 , 9-int(log10(Nb)) ) )
    MM = nint(Nb*(10**puiss))
    call pgnumb(MM , -puiss , 0 , String , Length)

  end subroutine string_of_real2

  !-------------------------------------------------------------------------------------
  function change_case(instring,dir) result(outstring)
  !-------------------------------------------------------------------------------------
  ! Switch from lower to upper case or vice-versa: From Jean Ballet
  ! instring : input string
  ! dir      : direction (1: to lower case; 2: to uppercase; 0: switch case)

    character(len=*), intent(in) :: instring
    integer, intent(in) :: dir
    character(len=len(instring)) :: outstring

  ! ASCII positions of lower case and upper case letters

    integer, parameter :: start_up  = iachar("A"), end_up  = iachar("Z"), &
                          start_low = iachar("a"), end_low = iachar("z"), &
                          offset = start_low - start_up

    integer :: length, icur, pos

    length = len(instring)
    outstring = instring
    do icur = 1, length
       pos = iachar(instring(icur:icur))
       if (pos >= start_up .and. pos <= end_up .and. dir < 2) then
          pos = pos + offset
       else if (pos >= start_low .and. pos <= end_low .and. dir /= 1) then
          pos = pos - offset
       endif
       outstring(icur:icur) = achar(pos)
    enddo

  end function change_case  

!******************************************************************************
! Masks those features in the spectrum, which are likely to be zero-order 
! spectra overlapping the target spectrum
!******************************************************************************
!-------------------------------------------------------------------------------
  subroutine MaskZeroFeatures(netRate, bkgRate, flux, netChannel, nFoundFeatures, nSpectra, ny, featureWindow)
!-------------------------------------------------------------------------------

    real(kind=single),dimension(:,:),pointer :: netRate , bkgRate
    real(kind=single),dimension(:,:),pointer :: flux, netChannel 
    integer, dimension(:), pointer :: nFoundFeatures
    integer,intent(in) :: nSpectra, ny
    integer :: j, i
    integer :: blueEdge, redEdge, windowHalfWidth, nFeatures
    integer :: verbFlag
    real(kind=single) :: maxValue, maxValue2, meanValue 
    integer(kind=int32), dimension(1:ny) :: maskarray
    integer, intent(in) :: featureWindow
    real(kind=single) :: averDeltaLambda, nAver, binningFactor
    !
    nFoundFeatures=0
    if (nSpectra .lt. 1) return
    windowHalfWidth=featureWindow

    blueEdge=1
    redEdge=ny
    verbFlag=1
    do i=1,nSpectra
       ! Find blueEdge
       do j=1,ny
          blueEdge=j
          if (flux(i,j) .gt. 0) exit
       enddo
       ! find red edge
       do j=blueEdge+1,ny
          redEdge=j
          if (flux(i,j) .eq. 0) exit
       enddo       
       ! Check the wavelength bin
       nAver=0.0
       averDeltaLambda=0.0
       do j=blueEdge,redEdge-2
          averDeltaLambda=averDeltaLambda+netChannel(i,j+1)-netChannel(i,j)
          nAver=nAver+1.0
       enddo
       if (nAver .gt. 1.0) averDeltaLambda=averDeltalambda/nAver
       binningFactor=1.0
       if (featureWindow .eq. 200) then ! UV-filter
          if (averDeltaLambda .gt. 4.5) binningFactor=2.0 ! binned UV-image
       endif
       if (featureWindow .eq. 80) then ! Visual filter
          if (averDeltaLambda .gt. 8.9) binningFactor=2.0 ! binned Visual image
       endif
       if (binningFactor .gt. 1) windowHalfWidth=windowHalfWidth/binningFactor

       call FindPossibleFeatures(netRate(i,:), windowHalfWidth, &
            maskArray, ny, blueEdge, redEdge, verbFlag, nFeatures)
       if (nFeatures .gt. 0) then
          verbFlag=0
          call RemovePossibleFeatures(netRate(i,:), bkgRate(i,:), flux(i,:), &
               maskArray, ny, verbFlag, blueEdge, redEdge)
          nFoundFeatures(i)=nFeatures
       endif
    enddo ! j=1,ny

  end subroutine MaskZeroFeatures

!******************************************************************
! Subroutine for finding possible features (zero-orders)
! sitting on the spectrum 
!******************************************************************
  subroutine FindPossibleFeatures (spectrumArray, windowHalfWidth, &
       maskArray, ny, blueEdge, redEdge, verbFlag, nFeatures) 
    integer, intent(in) :: ny
    integer, intent(in) :: blueEdge, redEdge
    integer, intent(in) :: windowHalfWidth
    real(kind=single), intent(inout), dimension(1:ny) :: spectrumArray
    integer, intent(in) :: verbFlag ! auxiliary flag for test output
    real(kind=single), dimension(:), pointer :: verticalStrip
    real(kind=single), dimension(:), pointer :: verticalStrip2
    real(kind=single), dimension(:), pointer :: convStrip
    real(kind=single), dimension(:), pointer :: convStrip2
    integer, intent(out) :: nFeatures
    integer(kind=int32), intent(out), dimension(1:ny) :: maskArray
    integer(kind=int32), dimension(1:ny) :: maskArray2
    integer :: i, j, k, nx
    integer :: iLeft, iRight, jBlue, jRed 
    real(kind=single) :: convValue, nMean
    integer :: leftOkay, rightOkay

    allocate(verticalStrip(1:ny))
    allocate(verticalStrip2(1:ny))
    allocate(convStrip(1:ny))
    allocate(convStrip2(1:ny))
    maskArray=1
    maskArray2=1
    verticalStrip=spectrumArray
    verticalStrip2=spectrumArray

    ! smooth the strip
    !-----------------------------------------------------
!    windowHalfWidth=50
    do j=1, ny
       convStrip(j)=smoothMedianWindow(verticalStrip, j, ny, windowHalfWidth)
    enddo
    do j=1, ny
       verticalStrip(j)=smoothAver(convStrip, j, ny)
    enddo
    !-----------------------------------------------------
    ! Compute maximum of the spectrum itself
    jBlue=blueEdge
    jRed=redEdge
    !-----------------------------
    verticalStrip2=verticalStrip2-verticalStrip
    where (verticalStrip2 .lt. 0.0) verticalStrip2=0.0
    !-----------------------------
    ! Initialise the mask
    do j=jBlue, jRed
       maskArray(j)=1
       maskArray2(j)=1
       if (verticalStrip2(j) .gt. verticalStrip(j)) then
          maskArray(j)=0
          maskArray2(j)=0
       endif
       !if (verbFlag .eq. 1) &
       !     write(*,*) " j=",j, " v=", verticalStrip(j), &
       !     " v2=", verticalStrip2(j), " mask=", maskArray(j)
    enddo

    ! Filter the mask 
    do j=jBlue, jRed
       ! check if there are more clean mask pixels
       if (maskArray(j) .eq. 0) then
          if (.not. maskArrayOkay(maskArray, ny, j, windowHalfWidth/10, leftOkay, rightOkay)) maskArray(j)=1  
       endif
    enddo

    !if (verbFlag .eq. 1) then
    !   do j=jBlue, jRed
    !      write(*,*) "L2313  j=",j, " v=", verticalStrip(j), &
    !           " v2=", verticalStrip2(j), " mask=", maskArray(j)
    !   enddo
    !endif
    nFeatures=0
    do j=jBlue+1,jRed
       if (maskArray(j) .eq. 0 .and. maskArray(j-1) .eq. 1) nFeatures=NFeatures+1
    enddo

    deallocate(verticalStrip)
    deallocate(verticalStrip2)
    deallocate(convStrip)
    deallocate(convStrip2)
  end subroutine FindPossibleFeatures

!******************************************************************
! Subroutine for finding possible features (zero-orders)
! sitting on the spectrum 
!******************************************************************
  subroutine RemovePossibleFeatures (spectrumArray, bkgArray, fluxArray, &
       maskArray, ny, verbFlag, blueEdge, redEdge) 
    integer, intent(in) :: ny
    integer, intent(in) :: blueEdge, redEdge
    real(kind=single), intent(inout), dimension(1:ny) :: spectrumArray
    real(kind=single), intent(inout), dimension(1:ny) :: bkgArray
    real(kind=single), intent(inout), dimension(1:ny) :: fluxArray
    integer(kind=int32), intent(in), dimension(1:ny) :: maskArray
    integer :: j
    integer, intent(in) :: verbFlag ! auxiliary flag for test output
    real(kind=single) :: maxArray, maxBkg, maxFlux
!
    ! Check if the amplitude of the features is larger than the continuum
    maxArray=0.0
    maxBkg=0.0
    maxFlux=0.0
    do j=blueEdge,redEdge
       if (.not. maskArray(j) .eq. 0) then
          if (spectrumArray(j) .gt. maxArray) maxArray=spectrumArray(j)
          if (bkgArray(j) .gt. maxBkg) maxBkg=bkgArray(j)
          if (fluxArray(j) .gt. maxFlux) maxFlux=fluxArray(j)
       endif
    enddo
    
    do j=blueEdge,redEdge
       if (maskArray(j) .eq. 0) then
          if (spectrumArray(j) .gt. maxArray) spectrumArray(j)=maxArray
          if (bkgArray(j) .gt. maxBkg) bkgArray(j)=maxBkg
          if (fluxArray(j) .gt. maxFlux) fluxArray(j)=maxFlux
       endif
    enddo
  end subroutine RemovePossibleFeatures

!********************************************************************
! Checking if the zeroed pixels in the mask array are surrounded by
! at least "nClearPixels" 
!********************************************************************
logical function MaskArrayOkay(maskArray, ny, j, nClearPixels, leftOkay, rightOkay)
  integer, intent(in) :: ny
  integer(int32), intent(in), dimension(1:ny)  :: maskArray
  integer, intent(in) :: j
  integer, intent(in) :: nClearPixels
  integer :: i
  integer, intent(out)  :: leftOkay, rightOkay
  MaskArrayOkay = .true.
  leftOkay=0
  rightOkay=0
  if (maskArray(j) .eq. 1) return
  do i=1,nClearPixels
     if (j+i .ge. ny) exit
     if (maskArray(j) .eq. 1) exit
     if (maskArray(j+i) .eq. 1) then
        exit
     else
        rightOkay=rightOkay+1
     endif
  enddo
  do i=1,nClearPixels
     if (j .eq. 1) exit
     if (j-i .lt. 1) exit
     if (maskArray(j-i) .eq. 1) then
        exit
     else
        leftOkay=leftOkay+1
     endif
  enddo
  if (leftOkay+rightOkay .lt. nClearPixels) MaskArrayOkay=.false.
end function MaskArrayOkay

!---smoothing by averaging
real function smoothAver(vect, j, nmax)
  integer, intent(in) :: nmax
  real(kind=single), intent(in) :: vect(1:nmax)
  integer, intent(in) :: j
  integer :: k
  real(kind=single) :: sum, nsum
  sum=0.0
  nsum=0.0
  do k=1,41
     if (j+k-21 .gt. 1 .and. j+k-21 .le. nmax) then
        sum=sum+vect(j+k-21)
        nsum=nsum+1.0
     endif
  enddo
  if (nsum .gt. 0.0) sum=sum/nsum
  smoothAver=sum  
end function smoothAver

! Median smoothing function within a given window
real function smoothMedianWindow(vect, j, nmax, windowHalfWidth)
  integer, intent(in) :: nmax
  real(kind=single), intent(in) :: vect(1:nmax)
  integer, intent(in) :: j
  integer, intent(in) :: windowHalfWidth
  real(kind=single) :: a(1:2*windowHalfWidth+1) ! smoothing for +-5 elements
  integer :: k,l
  real(kind=single) :: dum
  ! Copy the current 11 elements of the vector (indexed +-5)
  do k=1,2*windowHalfWidth+1 !j-5,j+5
     if (j+k-windowHalfWidth-1 .lt. 1 .or. j+k-windowHalfWidth-1 .gt. nmax) then
        a(k) = 9.E+20
     else
        a(k)=vect(j+k-windowHalfWidth-1)
     endif
  enddo
  ! ordering the vector a
  do k=1,2*windowHalfWidth+1
     do l=k+1, 2*windowHalfWidth+1
        if (a(l) .lt. a(k)) then
           dum=a(k)
           a(k)=a(l)
           a(l)=dum
        endif
     enddo
  enddo
  smoothMedianWindow=a(windowHalfWidth+1)
end function smoothMedianWindow
  
end module omgrismplot_mod




